/*
 * \file       trc_instr_decode_i.h
 * \brief      OpenCSD : Interface for instruction decode.
 * 
 * \copyright  Copyright (c) 2015, ARM Limited. All Rights Reserved.
 */

/* 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution. 
 * 
 * 3. Neither the name of the copyright holder nor the names of its contributors 
 * may be used to endorse or promote products derived from this software without 
 * specific prior written permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 'AS IS' AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND 
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */ 

#ifndef ARM_TRC_INSTR_DECODE_I_H_INCLUDED
#define ARM_TRC_INSTR_DECODE_I_H_INCLUDED

/*!
 * @class IInstrDecode   
 * @ingroup ocsd_interfaces
 * @brief Interface class to an instruction opcode decoder.
 *
 * The opcode decoder needs to be capable of limited decode required for trace
 * execution flow determination.
 *
 */
class IInstrDecode 
{
public:
    IInstrDecode() {}; /**< Default constructor. */
    virtual ~IInstrDecode() {}; /**< Default destructor. */

    /*!
     * Instruction opcode decode for the packet trace decoder to follow the
     * instruction execution flow. 
     *
     * @param *instr_info : Structure to pass current opcode, and receive required decode information.
     *
     * @return ocsd_err_t  : OCSD_OK if successful.
     */
    virtual ocsd_err_t DecodeInstruction(ocsd_instr_info *instr_info) = 0;
};

#endif // ARM_TRC_INSTR_DECODE_I_H_INCLUDED

/* End of File trc_instr_decode_i.h */
